
///////////////////////////////////////////////////////////////////////////////
// The MippinUpdate class gets the very latest content from Mippin

// Constructor.
function MippinUpdate() {
    this.httpReq = null;
    this.callback = null;
	this.channelOb = null;
}

// gets content for specified category and calls the callback when the feed
// has been fetched and parsed, or if the process results in an error.
MippinUpdate.prototype.fetchFeed = function(feedURL, callback,channelOb) {
	// remember callback
    this.callback = callback;
    this.channelOb = channelOb;
     if (!this.isOnline()) {
		this.callback.call(this, {
			status: "error"
		});
		return;
	}
	
	// create new XML HTTP request
    this.httpReq = new XMLHttpRequest();
    
    // set callback
    var self = this;
    this.httpReq.onreadystatechange = function() { self.readyStateChanged(); };

    // append the current time after the URL to bypass caches
    var fullURL = feedURL;
    if (fullURL.indexOf("?") == -1) {
        fullURL += "?";
    } else {
        fullURL += "&";
    }
    fullURL += "nocache=" + (new Date().getTime());
    
    // initiate the request
    try {
		this.httpReq.open("GET", fullURL, true);
		this.httpReq.send(null);
	} catch (err){
		this.callback.call(this,{ status: "error" });
	}
}

MippinUpdate.prototype.isOnline = function(adsURL, callback){
	if (window.widget && (typeof window.navigator.onLine == 'number')) {
		switch (window.navigator.onLine) {
			case window.navigator.NetworkAccessAllowed:{
				return true;
			}
			case window.navigator.NetworkAccessible:{
				return true;
			}
			case window.navigator.NetworkNotAllowed:{
				return false;
			}
			default:
				{
					return false;
				}
		}
	} else return true;
}


MippinUpdate.prototype.fetchAds = function(adsURL, callback) {
    // remember callback
    this.callback = callback;
    
    // create new XML HTTP request
    this.httpReq = new XMLHttpRequest();
    
    // set callback
    var self = this;
    if (!this.isOnline()) {
		this.callback.call(this, {
			status: "error"
		});
		return;
	}
   
	this.httpReq.onreadystatechange = function() { self.readyStateChangedAds(); };

    // append the current time after the URL to bypass caches
    var fullURL = adsURL;
    if (fullURL.indexOf("?") == -1) {
        fullURL += "?";
    } else {
        fullURL += "&";
    }
    fullURL += "nocache=" + (new Date().getTime());
    
    // initiate the request
    try {
		this.httpReq.open("GET", fullURL, true);
		this.httpReq.send(null);
	} catch (err){
		this.callback.call(this,{ status: "error" });
	}
}


MippinUpdate.prototype.abort = function(){
	// stop request
	if (this.httpReq.transport)
		this.httpReq.transport.abort();
}

// Callback for ready-state change events in the XML HTTP request.
MippinUpdate.prototype.readyStateChanged = function() {
    // complete request?
    if (this.httpReq.readyState == 4) {
        // attempt to get response status
        var responseStatus = null;
        try {
            responseStatus = this.httpReq.status;
        } catch (noStatusException) {}
        
        // handle the response and call the registered callback
        this.callback.call(this, this.handleResponse(responseStatus, this.httpReq.responseXML));
    }
}

MippinUpdate.prototype.readyStateChangedAds = function() {
    // complete request?
    if (this.httpReq.readyState == 4) {
        // attempt to get response status
        var responseStatus = null;
        try {
            responseStatus = this.httpReq.status;
        } catch (noStatusException) {}
        
        // handle the response and call the registered callback
        this.callback.call(this, this.handleResponseAds(responseStatus, this.httpReq.responseXML));
    }
}

// Handles a completed response.
MippinUpdate.prototype.handleResponse = function(responseStatus, xmlDoc) {
    if (responseStatus == 200 && xmlDoc != null) {
		// node ref for iterating
        var node;
		var myadnum=-1;
        var channelTitles =[];
		var channelCount=-1;
		var template = 0;
			
        // get last modified time - default to current time
        var lastModified = new Date().getTime();
        var channelElements = xmlDoc.getElementsByTagName("channel");
        if (channelElements.length > 0) {
            node = channelElements[0].firstChild;
            while (node != null) {
                if (node.nodeType == Node.ELEMENT_NODE) {
                    if (node.nodeName == "ads") {
                        myadnum = parseInt(this.getTextOfNode(node));
                    } else if (node.nodeName == "channelCount") {
                        channelCount = parseInt(this.getTextOfNode(node));
                    } else if (node.nodeName == "channelTitle") {
                        channelTitles.push(this.getTextOfNode(node));
                    } else if (node.nodeName == "pubDate" ||
                            node.nodeName == "lastBuildDate" ||
                            node.nodeName == "dc:date") {
                        	lastModified = this.getTextOfNode(node);
                    } else if (node.nodeName == "template") {
                        template = this.getTextOfNode(node);
                    }
                }
                node = node.nextSibling;
            }
        }
        
         var items = [];
        
        var itemElements = xmlDoc.getElementsByTagName("item");
        for (var i = 0; i < itemElements.length; i++) {
            var title = null;
            var date = null;
            var description = null;
			var mipSource=null;
            var url = null;
			var image = null;
			var imageThumb=null;
			var enclosure = null;
			var mippinTitle = null;
			var mippinID =null;
			var mippinUrl = null;
			var icon = null;
			var upgrade = null;
			var track = null;
			var advertHTML = null;
			var audio =[];
            
            node = itemElements[i].firstChild;
            while (node != null) {
                if (node.nodeType == Node.ELEMENT_NODE) {
                    if (node.nodeName == "title") {
                        title = this.getTextOfNode(node);
                    } else if (node.nodeName == "pubDate" || node.nodeName == "dc:date") {
                        date = this.getTextOfNode(node);
                    } else if (node.nodeName == "description") {
                        description = this.getHTMLOfNode(node);
                    } else if (node.nodeName == "link") {
                        url = this.getTextOfNode(node);
                    } else if (node.nodeName == "mippinlink") {
                        mippinUrl = this.getTextOfNode(node);
                    } else if (node.nodeName == "mippinSource") {
                        mipSource = this.getTextOfNode(node);
                    } else if (node.nodeName == "mippinTitle") {
                        mippinTitle = this.getTextOfNode(node);
                    } else if (node.nodeName == "upgrade") {
                        upgrade = this.getTextOfNode(node);
                    } else if (node.nodeName == "track") {
                        track = this.getTextOfNode(node);
                    } else if (node.nodeName == "icon") {
                        icon = this.getTextOfNode(node);
                    } else if (node.nodeName == "media:image") {
                        image = node.attributes.getNamedItem("url").nodeValue;
                    } else if (node.nodeName == "media:audio") {
                        audio.push(node.attributes.getNamedItem("url").nodeValue);
                    } else if (node.nodeName == "media:thumbnail") {
                        imageThumb = node.attributes.getNamedItem("url").nodeValue;
                    } else if (node.nodeName == "enclosure") {
                        enclosure = node.attributes.getNamedItem("url").nodeValue;
                    }
                }
                node = node.nextSibling;
            }
            
            // create the item and add to the items array
            items.push({ title: title, date: date, description: description, url: url, image: image, imageThumb: imageThumb, enclosure: enclosure, mippinTitle: mippinTitle, mippinUrl: mippinUrl, mipSource: mipSource, icon: icon, upgrade: upgrade, track: track, audio: audio});
        }
        
        // update was completed successfully
        return { status: "ok", lastModified: lastModified,channelCount:channelCount,channelTitles:channelTitles, adnum:myadnum, items: items, template: template  };

    } else {
        // update failed
        return { status: "error" };
    }
}

MippinUpdate.prototype.handleResponseAds = function(responseStatus, xmlDoc) {
    if (responseStatus == 200 && xmlDoc != null) {
        var node;
		var advertHTMLs = [];
        
       var itemElements = xmlDoc.getElementsByTagName("item");
        for (var i = 0; i < itemElements.length; i++) {
        	var advertHTML = null;
            
            node = itemElements[i].firstChild;
            while (node != null) {
                if (node.nodeType == Node.ELEMENT_NODE) {
                    if (node.nodeName == "advert-html") {
                        advertHTML = this.getHTMLOfNode(node);
                    } 
                }
                node = node.nextSibling;
            }
            
            advertHTMLs.push(advertHTML);
        }
        
        // update was completed successfully
        return { status: "ok", advertHTMLs: advertHTMLs };
    } else {
        // update failed
        return { status: "error" };
    }
}

// Returns the text of a node.
MippinUpdate.prototype.getTextOfNode = function(node) {
    var buf = "";
    
    // iterate through all child elements and collect all text to the buffer
    var child = node.firstChild;
    while (child != null) {
        if (child.nodeType == Node.TEXT_NODE || child.nodeType == Node.CDATA_SECTION_NODE) {
            // append text to buffer
            if (buf != "") {
                buf += " ";
            }
            buf += child.nodeValue;
        }
        child = child.nextSibling;
    }
    
    // strip all tags from the buffer
    var strippedBuf = "";
    var textStartPos = -1;
    var tagBalance = 0;
    
    // iterate through the text and append all text to the stripped buffer
    // that is at a tag balance of 0
    for (pos = 0; pos < buf.length; pos++) {
        var c = buf.charAt(pos);
        if (c == '<') {
            // entering a tag
            if (tagBalance == 0 && textStartPos != -1) {
                // everything up to here was valid text
                strippedBuf += buf.substring(textStartPos, pos);
                textStartPos = -1;
            }
            tagBalance++;
        } else if (c == '>') {
            // leaving a tag
            tagBalance--;
            textStartPos = -1;
        } else if (tagBalance == 0 && textStartPos == -1) {
            // first char of text
            textStartPos = pos;
        }
    }
    
    // add remaining text - if any
    if (tagBalance == 0 && textStartPos != -1) {
        strippedBuf += buf.substring(textStartPos, pos);
    }
    
    return strippedBuf;
}


MippinUpdate.prototype.getHTMLOfNode = function(node) {
    var buf = "";
    
    // iterate through all child elements and collect all text to the buffer
    var child = node.firstChild;
    while (child != null) {
        if (child.nodeType == Node.TEXT_NODE || child.nodeType == Node.CDATA_SECTION_NODE) {
            // append text to buffer
            if (buf != "") {
                buf += " ";
            }
            buf += child.nodeValue;
        }
        child = child.nextSibling;
    }
	return buf;    
}
